#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "sonnenverlauf.h"
#include "datum.h"

int strtod_error( const char *str, double *value )	{
	
	char *endptr;
	if ( *str == '\0' ) return 1;
	*value = strtod( str, &endptr );
	if ( *endptr != '\0' ) return 1;
	return 0;
}

double Mittelwert( double min, double max ) {
	
	return ( min + max ) / 2;
}

double modfi( double wert, int *integer ) {
	
	*integer = (int) wert;
	return wert - *integer;
}

int main( int argc, char *argv[] ) {
	
	Datum_t Datum;
	double Zeitzone = 1.0;
	Sonnenverlauf_t Sonnenverlauf;
	double GeographischeBreite;
	double GeographischeLaenge;
	int opt;
	int GeographischeBreiteGelesen = 0;
	int GeographischeLaengeGelesen = 0;
	double a_min, a_max, u_min, u_max;
	int Stunde, Minute;
	int init;
	
	while ( ( opt = getopt( argc, argv, "b:l:z:" ) ) != -1 ) {	
    	switch ( opt ) {
			case 'b':
				if ( strtod_error( optarg, &GeographischeBreite ) ) {
					fprintf( stderr, "Geographische Breite konnte nicht gelesen werden. Parameter -b ist fehlerhaft!\n" );
					return 1;
				}
				GeographischeBreiteGelesen = 1;
				break;
			case 'l':
				if ( strtod_error( optarg, &GeographischeLaenge ) ) {
					fprintf( stderr, "Geographische Laenge konnte nicht gelesen werden. Parameter -l ist fehlerhaft!\n" );
					return 1;
				}
				GeographischeLaengeGelesen = 1;
				break;
			case 'z':
				if ( strtod_error( optarg, &Zeitzone ) ) {
					fprintf( stderr, "Zeitzone konnte nicht gelesen werden. Parameter -z ist fehlerhaft!\n" );
					return 1;
				}
				break;
      		default: return 1;
      	}
	}
	
	if ( !GeographischeBreiteGelesen ) {
		fprintf( stderr, "Geographische Breite wurde nicht angegeben. Parameter -b fehlt.\n" );
		return 1;
	}

	if ( !GeographischeLaengeGelesen ) {
		fprintf( stderr, "Geographische Laenge wurde nicht angegeben. Parameter -l fehlt.!\n" );
		return 1;
	}

	for ( Datum.Monat = 1; Datum.Monat <= 12; Datum.Monat++ ) {
		for ( Datum.Tag = 1; Datum.Tag <= Monatstage[1][Datum.Monat - 1]; Datum.Tag++ ) {
			init = 1;
			for ( Datum.Jahr = 2022; Datum.Jahr < 2033; Datum.Jahr++ ) {
				if ( ( Datum.Monat == 29 ) && ( Datum.Monat == 2 ) && !Schaltjahr( Datum.Jahr ) ) {
					continue;
				}
				else {
					Sonnenverlauf = BerechneSonnenverlauf( Datum, Zeitzone, GeographischeBreite, GeographischeLaenge );
					if ( init ) {
						a_min = Sonnenverlauf.Aufgang;
						a_max = Sonnenverlauf.Aufgang;
						u_min = Sonnenverlauf.Untergang;
						u_max = Sonnenverlauf.Untergang;
						init = 0;
					}
					else {
						if ( Sonnenverlauf.Aufgang < a_min ) a_min = Sonnenverlauf.Aufgang;
						else if ( Sonnenverlauf.Aufgang > a_max ) a_max = Sonnenverlauf.Aufgang;
						if ( Sonnenverlauf.Aufgang < a_min ) a_min = Sonnenverlauf.Aufgang;
						else if ( Sonnenverlauf.Aufgang > a_max ) a_max = Sonnenverlauf.Aufgang;
						
						if ( Sonnenverlauf.Untergang < u_min ) u_min = Sonnenverlauf.Untergang;
						else if ( Sonnenverlauf.Untergang > u_max ) u_max = Sonnenverlauf.Untergang;
						if ( Sonnenverlauf.Untergang < u_min ) u_min = Sonnenverlauf.Untergang;
						else if ( Sonnenverlauf.Untergang > u_max ) u_max = Sonnenverlauf.Untergang;
					}
				}
			}

			Minute = round( 60 * modfi( Mittelwert( a_min, a_max ), &Stunde ) );
			printf( "%02d;", Stunde );
			printf( "%02d;", Minute );
			
			Minute = round( 60 * modfi( Mittelwert( u_min, u_max ), &Stunde ) );
			printf( "%02d;", Stunde );
			printf( "%02d;\n", Minute );
		}
	}

	return 0;
}

